# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProfileLicense(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'username': 'str',
        'type': 'str',
        'remaining': 'float'
    }

    attribute_map = {
        'username': 'username',
        'type': 'type',
        'remaining': 'remaining'
    }

    def __init__(self, username=None, type=None, remaining=None):  # noqa: E501
        """ProfileLicense - a model defined in Swagger"""  # noqa: E501
        self._username = None
        self._type = None
        self._remaining = None
        self.discriminator = None
        self.username = username
        self.type = type
        self.remaining = remaining

    @property
    def username(self):
        """Gets the username of this ProfileLicense.  # noqa: E501

        The user that is being given the license.  # noqa: E501

        :return: The username of this ProfileLicense.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this ProfileLicense.

        The user that is being given the license.  # noqa: E501

        :param username: The username of this ProfileLicense.  # noqa: E501
        :type: str
        """
        if username is None:
            raise ValueError("Invalid value for `username`, must not be `None`")  # noqa: E501

        self._username = username

    @property
    def type(self):
        """Gets the type of this ProfileLicense.  # noqa: E501

        The type of license, tickets are only good for one instance creation, whereas allocations are returned when instances are deleted.  # noqa: E501

        :return: The type of this ProfileLicense.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ProfileLicense.

        The type of license, tickets are only good for one instance creation, whereas allocations are returned when instances are deleted.  # noqa: E501

        :param type: The type of this ProfileLicense.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = ["ticket", "allocation"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def remaining(self):
        """Gets the remaining of this ProfileLicense.  # noqa: E501

        For tickets, the number of remaining tickets. For allocations, the maximum number of active instances allowed at one time.  # noqa: E501

        :return: The remaining of this ProfileLicense.  # noqa: E501
        :rtype: float
        """
        return self._remaining

    @remaining.setter
    def remaining(self, remaining):
        """Sets the remaining of this ProfileLicense.

        For tickets, the number of remaining tickets. For allocations, the maximum number of active instances allowed at one time.  # noqa: E501

        :param remaining: The remaining of this ProfileLicense.  # noqa: E501
        :type: float
        """
        if remaining is None:
            raise ValueError("Invalid value for `remaining`, must not be `None`")  # noqa: E501

        self._remaining = remaining

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProfileLicense, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProfileLicense):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
