# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DiscoverDescription(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version': 'str',
        'capailities': 'list[CapabilityDescription]',
        'end_points': 'list[DiscoverDescriptionEndPoints]'
    }

    attribute_map = {
        'version': 'version',
        'capailities': 'capailities',
        'end_points': 'end_points'
    }

    def __init__(self, version=None, capailities=None, end_points=None):  # noqa: E501
        """DiscoverDescription - a model defined in Swagger"""  # noqa: E501
        self._version = None
        self._capailities = None
        self._end_points = None
        self.discriminator = None
        if version is not None:
            self.version = version
        if capailities is not None:
            self.capailities = capailities
        if end_points is not None:
            self.end_points = end_points

    @property
    def version(self):
        """Gets the version of this DiscoverDescription.  # noqa: E501


        :return: The version of this DiscoverDescription.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this DiscoverDescription.


        :param version: The version of this DiscoverDescription.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def capailities(self):
        """Gets the capailities of this DiscoverDescription.  # noqa: E501


        :return: The capailities of this DiscoverDescription.  # noqa: E501
        :rtype: list[CapabilityDescription]
        """
        return self._capailities

    @capailities.setter
    def capailities(self, capailities):
        """Sets the capailities of this DiscoverDescription.


        :param capailities: The capailities of this DiscoverDescription.  # noqa: E501
        :type: list[CapabilityDescription]
        """

        self._capailities = capailities

    @property
    def end_points(self):
        """Gets the end_points of this DiscoverDescription.  # noqa: E501


        :return: The end_points of this DiscoverDescription.  # noqa: E501
        :rtype: list[DiscoverDescriptionEndPoints]
        """
        return self._end_points

    @end_points.setter
    def end_points(self, end_points):
        """Sets the end_points of this DiscoverDescription.


        :param end_points: The end_points of this DiscoverDescription.  # noqa: E501
        :type: list[DiscoverDescriptionEndPoints]
        """

        self._end_points = end_points

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DiscoverDescription, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DiscoverDescription):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
