# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProfileEdit(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'path': 'str',
        'valid': 'str',
        'description': 'str'
    }

    attribute_map = {
        'path': 'path',
        'valid': 'valid',
        'description': 'description'
    }

    def __init__(self, path=None, valid=None, description=None):  # noqa: E501
        """ProfileEdit - a model defined in Swagger"""  # noqa: E501
        self._path = None
        self._valid = None
        self._description = None
        self.discriminator = None
        self.path = path
        if valid is not None:
            self.valid = valid
        if description is not None:
            self.description = description

    @property
    def path(self):
        """Gets the path of this ProfileEdit.  # noqa: E501

        The user that is being given the license.  # noqa: E501

        :return: The path of this ProfileEdit.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this ProfileEdit.

        The user that is being given the license.  # noqa: E501

        :param path: The path of this ProfileEdit.  # noqa: E501
        :type: str
        """
        if path is None:
            raise ValueError("Invalid value for `path`, must not be `None`")  # noqa: E501

        self._path = path

    @property
    def valid(self):
        """Gets the valid of this ProfileEdit.  # noqa: E501

        The validation constraint itself. This can be a CSV, a numeric range, or raw regex (without slashes or flags).  # noqa: E501

        :return: The valid of this ProfileEdit.  # noqa: E501
        :rtype: str
        """
        return self._valid

    @valid.setter
    def valid(self, valid):
        """Sets the valid of this ProfileEdit.

        The validation constraint itself. This can be a CSV, a numeric range, or raw regex (without slashes or flags).  # noqa: E501

        :param valid: The valid of this ProfileEdit.  # noqa: E501
        :type: str
        """

        self._valid = valid

    @property
    def description(self):
        """Gets the description of this ProfileEdit.  # noqa: E501

        User-facing label for understanding field context.  # noqa: E501

        :return: The description of this ProfileEdit.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ProfileEdit.

        User-facing label for understanding field context.  # noqa: E501

        :param description: The description of this ProfileEdit.  # noqa: E501
        :type: str
        """

        self._description = description

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProfileEdit, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProfileEdit):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
